==> nivo-0.2.0.tgz: package/es/lib/charts/line/index.js <==
import { range, max, maxBy, sumBy, uniq } from 'lodash-es';
import { scalePoint, scaleLinear } from 'd3';

/**
 * Generates X scale.
 *
 * @param {Array.<Object>} data
 * @param {number}         width
 * @returns {Function}
 */
export var getXScale = function getXScale(data, width) {
    var xLengths = uniq(data.map(function (_ref) {
        var data = _ref.data;
        return data.length;
    }));
    if (xLengths.length > 1) {
        throw new Error(['Found inconsitent data for x,', 'expecting all series to have same length', 'but found: ' + xLengths.join(', ')].join(' '));
    }

    return scalePoint().range([0, width]).domain(data[0].data.map(function (_ref2) {
        var x = _ref2.x;
        return x;
    }));
};

/**
 * Generates Y scale for line chart.
 *
 * @param {Array.<Object>} data
 * @param {number}         height
 * @returns {Function}
 */
export var getYScale = function getYScale(data, height) {
    var maxY = maxBy(data.reduce(function (acc, serie) {
        return [].concat(acc, serie.data);
    }, []), 'y').y;

    return scaleLinear().rangeRound([height, 0]).domain([0, maxY]);
};

/**
 * Generates Y scale for stacked line chart.
 *
 * @param {Array.<Object>} data
 * @param {Object}         xScale
 * @param {number}         height
 */
export var getStackedYScale = function getStackedYScale(data, xScale, height) {
    var maxY = max(range(xScale.domain().length).map(function (i) {
        return sumBy(data, function (serie) {
            return serie.data[i].y;
        });
    }));

    return scaleLinear().rangeRound([height, 0]).domain([0, maxY]);
};

/**
 * Generates x/y scales & lines for line chart.
 *
 * @param {Array.<Object>} data
 * @param {number}         width
 * @param {number}         height
 * @param {Function}       color
 * @return {{ xScale: Function, yScale: Function, lines: Array.<Object> }}
 */
export var generateLines = function generateLines(data, width, height, color) {
    var xScale = getXScale(data, width);
    var yScale = getYScale(data, height);

    var lines = data.map(function (_ref3) {
        var id = _ref3.id,
            serie = _ref3.data;
        return {
            id: id,
            color: color(id),
            points: serie.map(function (d) {
                return Object.assign({}, d, {
                    value: d.y,
                    x: xScale(d.x),
                    y: yScale(d.y)
                });
            })
        };
    });

    return { xScale: xScale, yScale: yScale, lines: lines };
};

/**
 * Generates x/y scales & lines for stacked line chart.
 *
 * @param {Array.<Object>} data
 * @param {number}         width
 * @param {number}         height
 * @param {Function}       color
 * @return {{ xScale: Function, yScale: Function, lines: Array.<Object> }}
 */
export var generateStackedLines = function generateStackedLines(data, width, height, color) {
    var xScale = getXScale(data, width);
    var yScale = getStackedYScale(data, xScale, height);

    var lines = data.reduce(function (acc, serie, serieIndex) {
        var previousPoints = serieIndex === 0 ? null : acc[serieIndex - 1].points;

        var id = serie.id,
            serieData = serie.data;


        return [].concat(acc, [{
            id: id,
            color: color(serie),
            points: serieData.map(function (d, i) {
                if (!previousPoints) {
                    return Object.assign({}, d, {
                        value: d.y,
                        x: d.x,
                        y: d.y
                    });
                }

                return Object.assign({}, d, {
                    value: d.y,
                    x: d.x,
                    y: d.y + previousPoints[i].accY
                });
            }).map(function (d) {
                return {
                    key: d.x,
                    value: d.value,
                    accY: d.y,
                    x: xScale(d.x),
                    y: yScale(d.y)
                };
            })
        }]);
    }, []);

    return { xScale: xScale, yScale: yScale, lines: lines };
};

==> nivo-0.9.0.tgz: package/es/lib/charts/line/index.js <==
import { range, max, maxBy, sumBy, uniq } from 'lodash-es';
import { scalePoint, scaleLinear } from 'd3-scale';

/**
 * Generates X scale.
 *
 * @param {Array.<Object>} data
 * @param {number}         width
 * @returns {Function}
 */
export var getXScale = function getXScale(data, width) {
    var xLengths = uniq(data.map(function (_ref) {
        var data = _ref.data;
        return data.length;
    }));
    if (xLengths.length > 1) {
        throw new Error(['Found inconsitent data for x,', 'expecting all series to have same length', 'but found: ' + xLengths.join(', ')].join(' '));
    }

    return scalePoint().range([0, width]).domain(data[0].data.map(function (_ref2) {
        var x = _ref2.x;
        return x;
    }));
};

/**
 * Generates Y scale for line chart.
 *
 * @param {Array.<Object>} data
 * @param {number}         height
 * @returns {Function}
 */
export var getYScale = function getYScale(data, height) {
    var maxY = maxBy(data.reduce(function (acc, serie) {
        return [].concat(acc, serie.data);
    }, []), 'y').y;

    return scaleLinear().rangeRound([height, 0]).domain([0, maxY]);
};

/**
 * Generates Y scale for stacked line chart.
 *
 * @param {Array.<Object>} data
 * @param {Object}         xScale
 * @param {number}         height
 */
export var getStackedYScale = function getStackedYScale(data, xScale, height) {
    var maxY = max(range(xScale.domain().length).map(function (i) {
        return sumBy(data, function (serie) {
            return serie.data[i].y;
        });
    }));

    return scaleLinear().rangeRound([height, 0]).domain([0, maxY]);
};

export var getStackedScales = function getStackedScales(data, width, height) {
    var xScale = getXScale(data, width);
    var yScale = getStackedYScale(data, xScale, height);

    return { xScale: xScale, yScale: yScale };
};

export var getScales = function getScales(data, width, height) {
    var xScale = getXScale(data, width);
    var yScale = getYScale(data, height);

    return { xScale: xScale, yScale: yScale };
};

/**
 * Generates x/y scales & lines for line chart.
 *
 * @param {Array.<Object>} data
 * @param {Function}       xScale
 * @param {Function}       yScale
 * @param {Function}       color
 * @return {{ xScale: Function, yScale: Function, lines: Array.<Object> }}
 */
export var generateLines = function generateLines(data, xScale, yScale, color) {
    return data.map(function (serie) {
        var id = serie.id,
            serieData = serie.data;


        return {
            id: id,
            color: color(serie),
            points: serieData.map(function (d) {
                return Object.assign({}, d, {
                    value: d.y,
                    x: xScale(d.x),
                    y: yScale(d.y)
                });
            })
        };
    });
};

/**
 * Generates x/y scales & lines for stacked line chart.
 *
 * @param {Array.<Object>} data
 * @param {Function}       xScale
 * @param {Function}       yScale
 * @param {Function}       color
 * @return {{ xScale: Function, yScale: Function, lines: Array.<Object> }}
 */
export var generateStackedLines = function generateStackedLines(data, xScale, yScale, color) {
    return data.reduce(function (acc, serie, serieIndex) {
        var previousPoints = serieIndex === 0 ? null : acc[serieIndex - 1].points;

        var id = serie.id,
            serieData = serie.data;


        return [].concat(acc, [{
            id: id,
            color: color(serie),
            points: serieData.map(function (d, i) {
                if (!previousPoints) {
                    return Object.assign({}, d, {
                        value: d.y,
                        x: d.x,
                        y: d.y
                    });
                }

                return Object.assign({}, d, {
                    value: d.y,
                    x: d.x,
                    y: d.y + previousPoints[i].accY
                });
            }).map(function (d) {
                return {
                    key: d.x,
                    value: d.value,
                    accY: d.y,
                    x: xScale(d.x),
                    y: yScale(d.y)
                };
            })
        }]);
    }, []);
};
